package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.bean.TrdEcuBrandLabelBean;
import com.gic.spark.entity.bean.TrdEcuSalesLabelBean;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeRequest;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.Optional;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.List;

/**
 * @description:
 *消费次数
 * @author: wangxk
 * @date: 2020/5/6
 */
public class TagConsumptionTimeFilter extends AbstractTagConsumFilter{
    private static TagConsumptionTimeFilter instance;
    public static TagConsumptionTimeFilter getInstance(){
        if(null==instance){
            instance=new TagConsumptionTimeFilter();
        }
        return instance;
    }
    private TagConsumptionTimeFilter(){}
    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHiveBrandLabel);
        result.add(dataSourceHiveSalesLabel);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeRequest consumeRequest=(TagConsumeRequest)request;
        JavaRDD<TrdEcuSalesLabelBean> salesLabelRDD=MysqlRddManager.getPojoFromDataset(dataSourceHiveSalesLabel.getDatasetByEntId(enterpriseId), TrdEcuSalesLabelBean.class).javaRDD();
        JavaRDD<TrdEcuBrandLabelBean> brandLabelRDD=MysqlRddManager.getPojoFromDataset(dataSourceHiveBrandLabel.getDatasetByEntId(enterpriseId), TrdEcuBrandLabelBean.class).javaRDD();

        JavaRDD<Tuple2<TrdEcuSalesLabelBean,Optional<Iterable<TrdEcuBrandLabelBean>>>>labelRDD=salesLabelRDD.mapToPair(data->Tuple2.apply(data.getEcu_id(),data))
                                                                                                .leftOuterJoin(brandLabelRDD.mapToPair(data->Tuple2.apply(data.getEcu_id(),data)).groupByKey())
                                                                                                .map(data->data._2());

        JavaRDD<TrdEcuSalesLabelBean> consumeRDD=statisticsTypeHandle(labelRDD,consumeRequest);
        JavaRDD<Long>ecuRdd=consumeRDD.mapToPair(data-> Tuple2.apply(data.getEcu_id(),data.getOrder_times())).reduceByKey((x,y)->x+y)
                                        .mapPartitions(data->{
                                            List<Long>result=new ArrayList();
                                            while (data.hasNext()){
                                                Tuple2<Long,Integer>tp2=data.next();
                                                switch (consumeRequest.getNumberType()){
                                                    case gt:
                                                        if(tp2._2()>consumeRequest.getBeginNum()){
                                                            result.add(tp2._1());
                                                        }
                                                        break;
                                                    case gte:
                                                        if(tp2._2()>=consumeRequest.getBeginNum()){
                                                            result.add(tp2._1());
                                                        }
                                                        break;
                                                    case lt:
                                                        if(tp2._2()<consumeRequest.getEndNum()){
                                                            result.add(tp2._1());
                                                        }
                                                        break;
                                                    case lte:
                                                        if(tp2._2()<=consumeRequest.getEndNum()){
                                                            result.add(tp2._1());
                                                        }
                                                        break;
                                                    case eq:
                                                        if(tp2._2().intValue()==consumeRequest.getEqualNum()){
                                                            result.add(tp2._1());
                                                        }
                                                        break;
                                                    case between:
                                                        if(tp2._2()>=consumeRequest.getBeginNum()
                                                                &&tp2._2()<=consumeRequest.getEndNum()){
                                                            result.add(tp2._1());
                                                        }
                                                    default:break;
                                                }
                                            }
                                            return result.iterator();
                                        });
        return ecuRdd;
    }
}
