package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.bean.TrdVirtualOrderBean;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeAmountRequest;
import com.gic.spark.util.CommonUtil;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.Optional;
import org.apache.spark.sql.Row;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.List;

/**
 * @description:
 * @author: wangxk
 * @date: 2020/8/6
 */
public class TagHistoryConsumeTotalFilter extends AbstractTagConsumRecordFilter{

    private static TagHistoryConsumeTotalFilter instance;

    public static TagHistoryConsumeTotalFilter getInstance() {
        if(null==instance){
            instance=new TagHistoryConsumeTotalFilter();
        }
        return instance;
    }

    private TagHistoryConsumeTotalFilter(){}

    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHiveOrder);
        result.add(dataSourceHiveOrderItem);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeAmountRequest consumeAmountRequest=(TagConsumeAmountRequest)request;
        JavaRDD<TrdVirtualOrderBean>consumeRecordRDD= MysqlRddManager.getPojoFromDataset(dataSourceHiveOrder.getDatasetByEntId(enterpriseId), TrdVirtualOrderBean.class).javaRDD();

        JavaRDD<Row>virtualOrderItemRdd=dataSourceHiveOrderItem.getDatasetByEntId(enterpriseId).select("virtual_order_id","ent_brand_id").javaRDD();

        JavaRDD<Tuple2<TrdVirtualOrderBean,Optional<Iterable<String>>>>orderRdd=consumeRecordRDD.mapToPair(data->Tuple2.apply(data.getVirtual_id(),data))
                                                                                .leftOuterJoin(virtualOrderItemRdd.mapToPair(row->Tuple2.apply(row.getLong(0),row.getString(1))).groupByKey())
                                                                                .map(data->data._2());

        consumeRecordRDD=statisticsTypeHandle(orderRdd,consumeAmountRequest);
        int configStatus= CommonUtil.getConfigStatus(enterpriseId);

        JavaRDD<Long>ecuRdd=consumeRecordRDD.mapToPair(data-> Tuple2.apply(data.getEcu_id(),configStatus==1?data.getPaid_amt():data.getPay_amt()))
                                                .reduceByKey((x,y)->x+y)
                                                .filter(data->{
                                                    boolean result=false;
                                                    switch (consumeAmountRequest.getNumberType()){
                                                        case between:
                                                            if(data._2()>=consumeAmountRequest.getBeginNum()
                                                                    &&data._2()<=consumeAmountRequest.getEndNum()){
                                                                result=true;
                                                            }
                                                            break;
                                                        case lt:
                                                            if(data._2()<consumeAmountRequest.getEndNum()){
                                                                result=true;
                                                            }
                                                            break;
                                                        case gt:
                                                            if(data._2()>consumeAmountRequest.getBeginNum()){
                                                                result=true;
                                                            }
                                                            break;
                                                        case eq:
                                                            if(data._2()==consumeAmountRequest.getEqualNum()){
                                                                result=true;
                                                            }
                                                            break;
                                                        case lte:
                                                            if(data._2()<=consumeAmountRequest.getEndNum()){
                                                                result=true;
                                                            }
                                                            break;
                                                        case gte:
                                                            if(data._2()>=consumeAmountRequest.getBeginNum()){
                                                                result=true;
                                                            }
                                                            break;
                                                        default:break;
                                                    }
                                                    return result;
                                                }).map(data->data._1());
        return ecuRdd;
    }
}
